@@ -7,6 +7,7 @@ from typing import Optional
 import pandas as pd
 
 from psycop_feature_generation.loaders.raw.sql_load import sql_load
+from psycop_feature_generation.loaders.raw.utils import unpack_intervals
 from psycop_feature_generation.utils import data_loaders
 
 
@@ -15,6 +16,8 @@ def coercion_duration(
     coercion_type: Optional[str] = None,
     reason_for_coercion: Optional[str] = None,
     n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
 ) -> pd.DataFrame:
     """Load coercion data. By default returns entire coercion data view with
     duration in hours as the value column.
@@ -23,16 +26,17 @@ def coercion_duration(
         coercion_type (str): Type of coercion, e.g. 'tvangsindlæggelse', 'bæltefiksering'. Defaults to None. # noqa: DAR102
         reason_for_coercion (str): Reason for coercion, e.g. 'farlighed'. Defaults to None.
         n_rows: Number of rows to return. Defaults to None which returns entire coercion data view.
+        unpack_to_intervals: Unpack time interval to rows with set frequency (see below). Defaults to False.
+        unpack_freq:  unpack_freq: Frequency string by which the interval will be unpacked. Default to "D" (day). For e.g., 5 hours, write "5H".
 
     Returns:
         pd.DataFrame
     """
+
     coercion_discard = """('Døraflåsning', 'Personlig afskærmning over 24 timer', 'Koordinationsplan',
     'Udskrivningsaftale', 'Særlige dørlåse', 'Personlige alarm- og pejlesystemer', 'Andet' )"""
 
-    view = "[FOR_tvang_alt_hele_kohorten_inkl_2021_feb2022]"
-
-    sql = f"SELECT dw_ek_borger, datotid_start_sei, datotid_slut_sei, varighed_timer_sei, typetekst_sei FROM [fct].{view} WHERE datotid_start_sei IS NOT NULL AND typetekst_sei NOT IN {coercion_discard}"
+    sql = f"SELECT dw_ek_borger, datotid_start_sei, datotid_slut_sei, varighed_timer_sei, typetekst_sei FROM [fct].[FOR_tvang_alt_hele_kohorten_inkl_2021_feb2022] WHERE datotid_start_sei IS NOT NULL AND typetekst_sei NOT IN {coercion_discard}"
 
     if coercion_type and reason_for_coercion is None:
 
@@ -67,12 +71,22 @@ def coercion_duration(
     # Change NaNs to 0
     df["value"].fillna(0, inplace=True)
 
+    if unpack_to_intervals:
+        df = unpack_intervals(
+            df,
+            starttime_col="datotid_start_sei",
+            endtime_col="timestamp",
+            unpack_freq=unpack_freq,
+        )
+
     return df[["dw_ek_borger", "timestamp", "value"]].reset_index(drop=True)
 
 
 def _concatenate_coercion(
     coercion_types_list: list[dict[str, str]],
     n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
 ) -> pd.DataFrame:
     """Aggregate multiple types of coercion with multiple reasons into one
     column.
@@ -80,6 +94,8 @@ def _concatenate_coercion(
     Args:
         coercion_types_list (list): list of dictionaries containing a 'coercion_type' key and a 'reason_for_coercion' key. If keys not in dicts, they are set to None # noqa: DAR102
         n (int, optional): Number of rows to return. Defaults to None.
+        unpack_to_intervals: Unpack time interval to rows with set frequency (see below). Defaults to False.
+        unpack_freq:  unpack_freq: Frequency string by which the interval will be unpacked. Default to "D" (day). For e.g., 5 hours, write "5H".
 
     Returns:
         pd.DataFrame
@@ -100,6 +116,8 @@ def _concatenate_coercion(
             coercion_type=d["coercion_type"],
             reason_for_coercion=d["reason_for_coercion"],
             n_rows=n_rows,
+            unpack_to_intervals=unpack_to_intervals,
+            unpack_freq=unpack_freq,
         )
         for d in coercion_types_list
     ]
@@ -111,7 +129,11 @@ def _concatenate_coercion(
 
 
 @data_loaders.register("farlighed")
-def farlighed(n_rows: Optional[int] = None) -> pd.DataFrame:
+def farlighed(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     coercion_types_list = [
         {
             "reason_for_coercion": "Farlighed",
@@ -124,55 +146,91 @@ def farlighed(n_rows: Optional[int] = None) -> pd.DataFrame:
     return _concatenate_coercion(
         coercion_types_list=coercion_types_list,
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
-# Røde papir ved tvangsindlæggelse/tvangstilbageholdelse
+# Røde papirer ved tvangsindlæggelse/tvangstilbageholdelse
 @data_loaders.register("paa_grund_af_farlighed")
-def paa_grund_af_farlighed(n_rows: Optional[int] = None) -> pd.DataFrame:
+def paa_grund_af_farlighed(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         reason_for_coercion="På grund af farlighed",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
-# Gule papir ved tvangsindlæggelse/tvangstilbageholdelse
+# Gule papirer ved tvangsindlæggelse/tvangstilbageholdelse
 @data_loaders.register("af_helbredsmaessige_grunde")
-def af_helbredsmaessige_grunde(n_rows: Optional[int] = None) -> pd.DataFrame:
+def af_helbredsmaessige_grunde(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         reason_for_coercion=" Af helbredsmæssige grunde",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("urolig_tilstand")
-def urolig_tilstand(n_rows: Optional[int] = None) -> pd.DataFrame:
+def urolig_tilstand(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         reason_for_coercion="Urolig tilstand",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("anden_begrundelse")
-def anden_begrundelse(n_rows: Optional[int] = None) -> pd.DataFrame:
+def anden_begrundelse(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         reason_for_coercion="Anden begrundelse",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("naerliggende_eller_vaesentlig_fare_for_patienten_eller_andre")
-def naerliggende_fare(n_rows: Optional[int] = None) -> pd.DataFrame:
+def naerliggende_fare(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         reason_for_coercion="Nærliggende_eller_væsentlig_fare_for_patienten_eller_andre",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 # GENERAL TYPE (tabeltekst) ###
 # frihedsberøvelser
 @data_loaders.register("skema_1")
-def skema_1(n_rows: Optional[int] = None) -> pd.DataFrame:
+def skema_1(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     coercion_types_list = [
         {
             "coercion_type": "Tvangsindlæggelse",
@@ -185,12 +243,18 @@ def skema_1(n_rows: Optional[int] = None) -> pd.DataFrame:
     return _concatenate_coercion(
         coercion_types_list=coercion_types_list,
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 # tvangsbehandlinger
 @data_loaders.register("skema_2")
-def skema_2(n_rows: Optional[int] = None) -> pd.DataFrame:
+def skema_2(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     coercion_types_list = [
         {
             "coercion_type": "Af legemlig lidelse",
@@ -209,11 +273,17 @@ def skema_2(n_rows: Optional[int] = None) -> pd.DataFrame:
     return _concatenate_coercion(
         coercion_types_list=coercion_types_list,
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("skema_2_without_nutrition")
-def skema_2_without_nutrition(n_rows: Optional[int] = None) -> pd.DataFrame:
+def skema_2_without_nutrition(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     coercion_types_list = [
         {
             "coercion_type": "Af legemlig lidelse",
@@ -234,7 +304,11 @@ def skema_2_without_nutrition(n_rows: Optional[int] = None) -> pd.DataFrame:
 
 # magtanvendelse
 @data_loaders.register("skema_3")
-def skema_3(n_rows: Optional[int] = None) -> pd.DataFrame:
+def skema_3(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     coercion_types_list = [
         {
             "coercion_type": "Bælte",
@@ -258,6 +332,8 @@ def skema_3(n_rows: Optional[int] = None) -> pd.DataFrame:
     return _concatenate_coercion(
         coercion_types_list=coercion_types_list,
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
@@ -266,88 +342,154 @@ def skema_3(n_rows: Optional[int] = None) -> pd.DataFrame:
 
 
 @data_loaders.register("baelte")
-def baelte(n_rows: Optional[int] = None) -> pd.DataFrame:
+def baelte(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Bælte",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("remme")
-def remme(n_rows: Optional[int] = None) -> pd.DataFrame:
+def remme(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Remme",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("fastholden")
-def fastholden(n_rows: Optional[int] = None) -> pd.DataFrame:
+def fastholden(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Fastholden",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("beroligende_medicin")
-def beroligende_medicin(n_rows: Optional[int] = None) -> pd.DataFrame:
+def beroligende_medicin(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Beroligende medicin",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("handsker")
-def handsker(n_rows: Optional[int] = None) -> pd.DataFrame:
+def handsker(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Handsker",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("tvangsindlaeggelse")
-def tvangsindlaeggelse(n_rows: Optional[int] = None) -> pd.DataFrame:
+def tvangsindlaeggelse(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Tvangsindlæggelse",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("tvangstilbageholdelse")
-def tvangstilbageholdelse(n_rows: Optional[int] = None) -> pd.DataFrame:
+def tvangstilbageholdelse(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Tvangstilbageholdelse",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("medicinering")
-def medicinering(n_rows: Optional[int] = None) -> pd.DataFrame:
+def medicinering(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Medicinering",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("ect")
-def ect(n_rows: Optional[int] = None) -> pd.DataFrame:
+def ect(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="ECT",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("ernaering")
-def ernaering(n_rows: Optional[int] = None) -> pd.DataFrame:
+def ernaering(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Ernæring",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
 
 
 @data_loaders.register("af_legemlig_lidelse")
-def af_legemlig_lidelse(n_rows: Optional[int] = None) -> pd.DataFrame:
+def af_legemlig_lidelse(
+    n_rows: Optional[int] = None,
+    unpack_to_intervals: Optional[bool] = False,
+    unpack_freq: Optional[str] = "D",
+) -> pd.DataFrame:
     return coercion_duration(
         coercion_type="Af legemlig lidelse",
         n_rows=n_rows,
+        unpack_to_intervals=unpack_to_intervals,
+        unpack_freq=unpack_freq,
     )
